# -*- coding: utf-8 -*-
from plone.app.relationfield.testing import FUNCTIONAL_WIDGETS_TESTING
from plone.testing import layered

import doctest
import os
import re
import six
import unittest


optionflags = doctest.ELLIPSIS | doctest.NORMALIZE_WHITESPACE


class Py23DocChecker(doctest.OutputChecker):
    def check_output(self, want, got, optionflags):
        if six.PY2:
            got = re.sub("u'(.*?)'", "'\\1'", got)
        return doctest.OutputChecker.check_output(self, want, got, optionflags)


def test_suite():
    suite = unittest.TestSuite()
    suite.addTests(
        [
            layered(
                doctest.DocFileSuite(
                    os.path.join(os.path.pardir, 'supermodel.txt'),
                    optionflags=optionflags,
                    checker=Py23DocChecker(),
                ),
                layer=FUNCTIONAL_WIDGETS_TESTING,
            )
        ]
    )
    return suite


if __name__ == '__main__':
    unittest.main(default='test_suite')



# -*- coding: utf-8 -*-
from plone.testing import layered
from plone.testing.zca import ZCMLSandbox
from unittest import TestSuite

import doctest
import plone.app.relationfield.tests


ZCML_SANDBOX = ZCMLSandbox(
    filename='test_marshall.zcml', package=plone.app.relationfield.tests
)


def test_suite():
    suite = TestSuite()
    OPTIONFLAGS = doctest.ELLIPSIS | doctest.NORMALIZE_WHITESPACE
    suite.addTest(
        layered(
            doctest.DocFileSuite(
                '../marshaler.rst',
                optionflags=OPTIONFLAGS,
                package='plone.app.relationfield.tests',
            ),
            layer=ZCML_SANDBOX,
        )
    )
    return suite



# -*- coding: utf-8 -*-
from five.intid.intid import IntIds
from five.intid.site import addUtility
from z3c.relationfield.index import RelationCatalog
from z3c.relationfield.interfaces import IRelationValue
from zc.relation.interfaces import ICatalog
from zope.intid.interfaces import IIntIds

import BTrees


PLONE_RELATION_INDEXES = [
    {'element': IRelationValue['from_id']},
    {'element': IRelationValue['to_id']},
    {
        'element': IRelationValue['from_attribute'],
        'kwargs': {'btree': BTrees.family32.OI},
    },
]


def relation_catalog_factory():
    return RelationCatalog(indexes=PLONE_RELATION_INDEXES)


def add_relations(context):
    addUtility(
        context,
        ICatalog,
        relation_catalog_factory,
        ofs_name='relations',
        findroot=False,
    )


def add_intids(context):
    addUtility(context, IIntIds, IntIds, ofs_name='intids', findroot=False)


def installRelations(context):
    if context.readDataFile('install_relations.txt') is None:
        return
    portal = context.getSite()
    add_relations(portal)
    return 'Added relations utility.'



# -*- coding: utf-8 -*-
from persistent import Persistent
from plone.app.relationfield import HAS_CONTENTTYPES
from plone.app.testing import FunctionalTesting
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from z3c.relationfield import RelationList
from z3c.relationfield.interfaces import IHasRelations
from zope.interface import implementer
from zope.interface import Interface

import zope.schema


if HAS_CONTENTTYPES:
    from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_FIXTURE


class IAddress(Interface):
    streetname = zope.schema.TextLine(title=u'Street name')
    city = zope.schema.TextLine(title=u'City')


@implementer(IAddress, IHasRelations)
class Address(Persistent):
    __name__ = u''
    streetname = u''
    city = u''

    def __init__(self, streetname, city):
        self.streetname = streetname
        self.city = city
        __name__ = '{streetname} - {city}'.format(**locals())


class IPerson(zope.interface.Interface):
    name = zope.schema.TextLine(title=u'Name', default=u'<no name>')
    phone = zope.schema.TextLine(title=u'Phone')
    addresses = RelationList(title=u'Addresses')


@implementer(IPerson, IHasRelations)
class Person(Persistent):
    name = u''

    def __init__(self, name):
        self.name = name


class PloneAppRelationfieldFixture(PloneSandboxLayer):
    if HAS_CONTENTTYPES:
        defaultBases = (PLONE_APP_CONTENTTYPES_FIXTURE,)
    else:
        defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        import plone.app.relationfield

        self.loadZCML(package=plone.app.relationfield)

    def setUpPloneSite(self, portal):
        self.applyProfile(portal, 'plone.app.relationfield:default')


FIXTURE = PloneAppRelationfieldFixture()
FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(FIXTURE,), name='plone.app.relationfield:Functional'
)


class PloneAppRelationfieldContentTreeFixture(PloneSandboxLayer):

    if HAS_CONTENTTYPES:
        defaultBases = (PLONE_APP_CONTENTTYPES_FIXTURE,)
    else:
        defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        import plone.app.dexterity

        self.loadZCML(package=plone.app.dexterity)

        import plone.formwidget.contenttree

        self.loadZCML(package=plone.formwidget.contenttree)

        import plone.app.relationfield

        self.loadZCML(package=plone.app.relationfield)

    def setUpPloneSite(self, portal):
        self.applyProfile(portal, 'plone.app.dexterity:default')
        self.applyProfile(portal, 'plone.formwidget.contenttree:default')
        self.applyProfile(portal, 'plone.app.relationfield:default')


CONTENTTREE_FIXTURE = PloneAppRelationfieldContentTreeFixture()

FUNCTIONAL_CONTENTTREE_TESTING = FunctionalTesting(
    bases=(CONTENTTREE_FIXTURE,),
    name='plone.app.relationfield.contenttree:Functional',
)


class PloneAppRelationfieldWidgetsFixture(PloneSandboxLayer):

    if HAS_CONTENTTYPES:
        defaultBases = (PLONE_APP_CONTENTTYPES_FIXTURE,)
    else:
        defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        import plone.app.dexterity

        self.loadZCML(package=plone.app.dexterity)

        import plone.app.relationfield

        self.loadZCML(package=plone.app.relationfield)

    def setUpPloneSite(self, portal):
        self.applyProfile(portal, 'plone.app.dexterity:default')
        self.applyProfile(portal, 'plone.app.relationfield:default')


WIDGETS_FIXTURE = PloneAppRelationfieldWidgetsFixture()

FUNCTIONAL_WIDGETS_TESTING = FunctionalTesting(
    bases=(WIDGETS_FIXTURE,),
    name='plone.app.relationfield.contenttree:Functional',
)



# -*- coding: utf-8 -*-
from setuptools import find_packages
from setuptools import setup


version = '3.0.1'

setup(
    name='plone.app.relationfield',
    version=version,
    description='Plone support for z3c.relationfield',
    long_description=(
        open('README.rst').read() + '\n' + open('CHANGES.rst').read()
    ),
    # Get more strings from
    # https://pypi.org/classifiers/
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Framework :: Plone',
        'Framework :: Plone :: 6.0',
        'Framework :: Plone :: Core',
        "License :: OSI Approved :: GNU General Public License (GPL)",
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    keywords='dexterity relations plone zc.relation',
    author='Alec Mitchell',
    author_email='apm13@columbia.edu',
    url='https://pypi.org/project/plone.app.relationfield',
    license='GPL',
    packages=find_packages(),
    namespace_packages=['plone', 'plone.app'],
    include_package_data=True,
    zip_safe=False,
    python_requires=">=3.8",
    install_requires=[
        'setuptools',
        'zope.interface',
        'zope.component',
        'zope.schema',
        'zope.intid',
        'five.intid',
        'plone.app.intid',
        'z3c.form',
        'z3c.relationfield>0.7.999',
        'z3c.formwidget.query',
        'plone.autoform',
        'plone.supermodel',
        'plone.app.vocabularies',
        'plone.schemaeditor>=1.3.5',
        'Products.CMFCore',
        'plone.rfc822',
        'plone.app.z3cform>=1.1.0',
    ],
    extras_require={
        'test': [
            'plone.app.testing',
            'plone.app.dexterity',
            'plone.app.robotframework',
        ]
    },
    entry_points="""
    # -*- Entry points: -*-
    """,
)



# -*- coding: utf-8 -*-
from plone.app.vocabularies.catalog import CatalogSource
from plone.supermodel.exportimport import BaseHandler
from plone.supermodel.utils import valueToElement
from z3c.relationfield.schema import RelationChoice
from z3c.relationfield.schema import RelationList
from zope import schema


class RelationChoiceBaseHandler(BaseHandler):

    filteredAttributes = BaseHandler.filteredAttributes.copy()
    filteredAttributes.update(
        {
            'portal_type': 'w',
            'source': 'rw',
            'vocabulary': 'rw',
            'vocabularyName': 'rw',
        }
    )

    def __init__(self, klass):
        super(RelationChoiceBaseHandler, self).__init__(klass)

        self.fieldAttributes['portal_type'] = schema.List(
            __name__='portal_type',
            title=u'Allowed target types',
            value_type=schema.Text(title=u'Type'),
        )

    def _constructField(self, attributes):
        portal_type = (
            attributes.get('portal_type')
            or attributes.get('portal_types')
            or []
        )
        if 'portal_type' in attributes:
            del attributes['portal_type']

        if not portal_type:
            attributes['source'] = CatalogSource()
        else:
            attributes['source'] = CatalogSource(portal_type=portal_type)

        return super(RelationChoiceBaseHandler, self)._constructField(
            attributes
        )

    def write(self, field, name, type, elementName='field'):
        element = super(RelationChoiceBaseHandler, self).write(
            field, name, type, elementName
        )
        portal_type = []

        portal_type.extend(field.source.query.get('portal_type') or [])

        if portal_type:
            attributeField = self.fieldAttributes['portal_type']
            child = valueToElement(
                attributeField, portal_type, name='portal_type', force=True
            )
            element.append(child)

        return element


RelationChoiceHandler = RelationChoiceBaseHandler(RelationChoice)
RelationListHandler = BaseHandler(RelationList)
